@@ -22,30 +22,28 @@
 The interface to execute commands on the command graph
 """
 
+from __future__ import annotations
+
 import traceback
 from abc import ABCMeta, abstractmethod
-from typing import Any, Dict, List, Tuple, Union
+from typing import TYPE_CHECKING
 
 from libqtile import ipc
-from libqtile.command.base import (
-    CommandError,
-    CommandException,
-    CommandObject,
-    SelectError,
-)
-from libqtile.command.graph import (
-    CommandGraphCall,
-    CommandGraphNode,
-    SelectorType,
-)
+from libqtile.command.base import CommandError, CommandException, CommandObject, SelectError
+from libqtile.command.graph import CommandGraphCall, CommandGraphNode
 from libqtile.log_utils import logger
 
+if TYPE_CHECKING:
+    from typing import Any
+
+    from libqtile.command.graph import SelectorType
+
 SUCCESS = 0
 ERROR = 1
 EXCEPTION = 2
 
 
-def format_selectors(selectors: List[SelectorType]) -> str:
+def format_selectors(selectors: list[SelectorType]) -> str:
     """Build the path to the selected command graph node"""
     path_elements = []
     for name, selector in selectors:
@@ -65,7 +63,7 @@ class CommandInterface(metaclass=ABCMeta):
     """
 
     @abstractmethod
-    def execute(self, call: CommandGraphCall, args: Tuple, kwargs: Dict) -> Any:
+    def execute(self, call: CommandGraphCall, args: tuple, kwargs: dict) -> Any:
         """Execute the given call, returning the result of the execution
 
         Perform the given command graph call, calling the function with the
@@ -87,9 +85,9 @@ class CommandInterface(metaclass=ABCMeta):
 
         Parameters
         ----------
-        node : CommandGraphNode
+        node: CommandGraphNode
             The node to check for commands
-        command : str
+        command: str
             The name of the command to check for
 
         Returns
@@ -99,16 +97,16 @@ class CommandInterface(metaclass=ABCMeta):
         """
 
     @abstractmethod
-    def has_item(self, node: CommandGraphNode, object_type: str, item: Union[str, int]) -> bool:
+    def has_item(self, node: CommandGraphNode, object_type: str, item: str | int) -> bool:
         """Check if the given item exists
 
         Parameters
         ----------
-        node : CommandGraphNode
+        node: CommandGraphNode
             The node to check for items
-        object_type : str
+        object_type: str
             The type of object to check for items.
-        command : str
+        command: str
             The name of the item to check for
 
         Returns
@@ -126,13 +124,13 @@ class QtileCommandInterface(CommandInterface):
 
         Parameters
         ----------
-        command_object : CommandObject
+        command_object: CommandObject
             The command object to use for resolving the commands and items
             against.
         """
         self._command_object = command_object
 
-    def execute(self, call: CommandGraphCall, args: Tuple, kwargs: Dict) -> Any:
+    def execute(self, call: CommandGraphCall, args: tuple, kwargs: dict) -> Any:
         """Execute the given call, returning the result of the execution
 
         Perform the given command graph call, calling the function with the
@@ -148,23 +146,26 @@ class QtileCommandInterface(CommandInterface):
             The keyword arguments to pass into the command graph call.
         """
         obj = self._command_object.select(call.selectors)
-
+        cmd = None
         try:
             cmd = obj.command(call.name)
         except SelectError:
+            pass
+
+        if cmd is None:
             return "No such command."
 
         logger.debug("Command: %s(%s, %s)", call.name, args, kwargs)
-        return cmd(*args, **kwargs)
+        return cmd(self._command_object, *args, **kwargs)
 
     def has_command(self, node: CommandGraphNode, command: str) -> bool:
         """Check if the given command exists
 
         Parameters
         ----------
-        node : CommandGraphNode
+        node: CommandGraphNode
             The node to check for commands
-        command : str
+        command: str
             The name of the command to check for
 
         Returns
@@ -176,16 +177,16 @@ class QtileCommandInterface(CommandInterface):
         cmd = obj.command(command)
         return cmd is not None
 
-    def has_item(self, node: CommandGraphNode, object_type: str, item: Union[str, int]) -> bool:
+    def has_item(self, node: CommandGraphNode, object_type: str, item: str | int) -> bool:
         """Check if the given item exists
 
         Parameters
         ----------
-        node : CommandGraphNode
+        node: CommandGraphNode
             The node to check for items
-        object_type : str
+        object_type: str
             The type of object to check for items.
-        item : str
+        item: str
             The name or index of the item to check for
 
         Returns
@@ -208,12 +209,12 @@ class IPCCommandInterface(CommandInterface):
 
         Parameters
         ----------
-        ipc_client : ipc.Client
+        ipc_client: ipc.Client
             The client that is to be used to resolve the calls.
         """
         self._client = ipc_client
 
-    def execute(self, call: CommandGraphCall, args: Tuple, kwargs: Dict) -> Any:
+    def execute(self, call: CommandGraphCall, args: tuple, kwargs: dict) -> Any:
         """Execute the given call, returning the result of the execution
 
         Executes the given command over the given IPC client.  Returns the
@@ -228,9 +229,7 @@ class IPCCommandInterface(CommandInterface):
         kwargs:
             The keyword arguments to pass into the command graph call.
         """
-        status, result = self._client.send((
-            call.parent.selectors, call.name, args, kwargs
-        ))
+        status, result = self._client.send((call.parent.selectors, call.name, args, kwargs))
         if status == SUCCESS:
             return result
         if status == ERROR:
@@ -245,9 +244,9 @@ class IPCCommandInterface(CommandInterface):
 
         Parameters
         ----------
-        node : CommandGraphNode
+        node: CommandGraphNode
             The node to check for commands
-        command : str
+        command: str
             The name of the command to check for
 
         Returns
@@ -259,7 +258,7 @@ class IPCCommandInterface(CommandInterface):
         commands = self.execute(cmd_call, (), {})
         return command in commands
 
-    def has_item(self, node: CommandGraphNode, object_type: str, item: Union[str, int]) -> bool:
+    def has_item(self, node: CommandGraphNode, object_type: str, item: str | int) -> bool:
         """Check if the given item exists
 
         Resolves the available commands for the given command node of the given
@@ -268,11 +267,11 @@ class IPCCommandInterface(CommandInterface):
 
         Parameters
         ----------
-        node : CommandGraphNode
+        node: CommandGraphNode
             The node to check for items
-        object_type : str
+        object_type: str
             The type of object to check for items.
-        command : str
+        command: str
             The name of the item to check for
 
         Returns
@@ -291,12 +290,12 @@ class IPCCommandServer:
     def __init__(self, qtile) -> None:
         """Wrapper around the ipc server for communitacing with the IPCCommandInterface
 
-        Sets up the IPC server such that it will receive and send messages to
+        sets up the IPC server such that it will receive and send messages to
         and from the IPCCommandInterface.
         """
         self.qtile = qtile
 
-    def call(self, data: Tuple[List[SelectorType], str, Tuple, Dict]) -> Tuple[int, Any]:
+    def call(self, data: tuple[list[SelectorType], str, tuple, dict]) -> tuple[int, Any]:
         """Receive and parse the given data"""
         selectors, name, args, kwargs = data
         try:
@@ -310,7 +309,12 @@ class IPCCommandServer:
 
         logger.debug("Command: %s(%s, %s)", name, args, kwargs)
         try:
-            return SUCCESS, cmd(*args, **kwargs)
+            # Check if method is bound
+            if hasattr(cmd, "__self__"):
+                return SUCCESS, cmd(*args, **kwargs)
+            else:
+                # If not, pass object as first argument
+                return SUCCESS, cmd(obj, *args, **kwargs)
         except CommandError as err:
             return ERROR, err.args[0]
         except Exception:
