@@ -26,11 +26,18 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
 from libqtile import hook, utils
 from libqtile.backend.base import FloatStates
-from libqtile.command.base import CommandObject, ItemT
+from libqtile.command.base import CommandObject, expose_command
 from libqtile.log_utils import logger
 
+if TYPE_CHECKING:
+    from libqtile.command.base import ItemT
+
 
 class _Group(CommandObject):
     """A container for a bunch of windows
@@ -40,11 +47,14 @@ class _Group(CommandObject):
 
     A group is identified by its name but displayed in GroupBox widget by its label.
     """
-    def __init__(self, name, layout=None, label=None):
+
+    def __init__(self, name, layout=None, label=None, screen_affinity=None):
+        self.screen_affinity = screen_affinity
         self.name = name
         self.label = name if label is None else label
         self.custom_layout = layout  # will be set on _configure
         self.windows = []
+        self.tiled_windows = set()
         self.qtile = None
         self.layouts = []
         self.floating_layout = None
@@ -56,6 +66,7 @@ class _Group(CommandObject):
         self.focus_history = []
         self.screen = None
         self.current_layout = None
+        self.last_focused = None
 
     def _configure(self, layouts, floating_layout, qtile):
         self.screen = None
@@ -112,14 +123,10 @@ class _Group(CommandObject):
         for index, obj in enumerate(self.layouts):
             if obj.name == layout:
                 self.current_layout = index
-                hook.fire(
-                    "layout_change",
-                    self.layouts[self.current_layout],
-                    self
-                )
+                hook.fire("layout_change", self.layouts[self.current_layout], self)
                 self.layout_all()
                 return
-        logger.error("No such layout: {}".format(layout))
+        logger.error("No such layout: %s", layout)
 
     def use_layout(self, index):
         assert -len(self.layouts) <= index < len(self.layouts), "layout index out of bounds"
@@ -145,22 +152,21 @@ class _Group(CommandObject):
         if self.screen and self.windows:
             with self.qtile.core.masked():
                 normal = [x for x in self.windows if not x.floating]
-                floating = [
-                    x for x in self.windows
-                    if x.floating and not x.minimized
-                ]
+                floating = [x for x in self.windows if x.floating and not x.minimized]
                 screen_rect = self.screen.get_rect()
                 if normal:
                     try:
                         self.layout.layout(normal, screen_rect)
                     except Exception:
-                        logger.exception("Exception in layout %s",
-                                         self.layout.name)
+                        logger.exception("Exception in layout %s", self.layout.name)
                 if floating:
                     self.floating_layout.layout(floating, screen_rect)
-                if self.current_window and \
-                        self.screen == self.qtile.current_screen:
+                if self.current_window and self.screen == self.qtile.current_screen:
                     self.current_window.focus(warp)
+                else:
+                    # Screen has lost focus so we reset record of focused window so
+                    # focus will warp when screen is focused again
+                    self.last_focused = None
 
     def set_screen(self, screen, warp=True):
         """Set this group's screen to screen"""
@@ -208,7 +214,13 @@ class _Group(CommandObject):
         if win:
             if win not in self.windows:
                 return
+
+            # ignore focus events if window is the current window
+            if win is self.last_focused:
+                warp = False
+
             self.current_window = win
+            self.last_focused = self.current_window
             if win.floating:
                 for layout in self.layouts:
                     layout.blur()
@@ -220,17 +232,20 @@ class _Group(CommandObject):
             hook.fire("focus_change")
             self.layout_all(warp)
 
+    @expose_command()
     def info(self):
+        """Returns a dictionary of info for this group"""
         return dict(
             name=self.name,
             label=self.label,
             focus=self.current_window.name if self.current_window else None,
+            tiled_windows={i.name for i in self.tiled_windows},
             windows=[i.name for i in self.windows],
             focus_history=[i.name for i in self.focus_history],
             layout=self.layout.name,
             layouts=[i.name for i in self.layouts],
             floating_info=self.floating_layout.info(),
-            screen=self.screen.index if self.screen else None
+            screen=self.screen.index if self.screen else None,
         )
 
     def add(self, win, focus=True, force=False):
@@ -240,13 +255,16 @@ class _Group(CommandObject):
         win.group = self
         if self.qtile.config.auto_fullscreen and win.wants_to_fullscreen:
             win._float_state = FloatStates.FULLSCREEN
-        elif self.floating_layout.match(win):
+        elif self.floating_layout.match(win) and not win.fullscreen:
             win._float_state = FloatStates.FLOATING
-        if win.floating:
-            self.floating_layout.add(win)
-        else:
+            if self.qtile.config.floats_kept_above:
+                win.keep_above(enable=True)
+        if win.floating and not win.fullscreen:
+            self.floating_layout.add_client(win)
+        if not win.floating or win.fullscreen:
+            self.tiled_windows.add(win)
             for i in self.layouts:
-                i.add(win)
+                i.add_client(win)
         if focus:
             self.focus(win, warp=True, force=force)
 
@@ -258,21 +276,29 @@ class _Group(CommandObject):
         if win.floating:
             nextfocus = self.floating_layout.remove(win)
 
-            nextfocus = nextfocus or \
-                self.current_window or \
-                self.layout.focus_first() or \
-                self.floating_layout.focus_first(group=self)
-        else:
+            nextfocus = (
+                nextfocus
+                or self.current_window
+                or self.layout.focus_first()
+                or self.floating_layout.focus_first(group=self)
+            )
+        # Remove from the tiled layouts if it was not floating or fullscreen
+        if not win.floating or win.fullscreen:
             for i in self.layouts:
                 if i is self.layout:
                     nextfocus = i.remove(win)
                 else:
                     i.remove(win)
 
-            nextfocus = nextfocus or \
-                self.floating_layout.focus_first(group=self) or \
-                self.current_window or \
-                self.layout.focus_first()
+            nextfocus = (
+                nextfocus
+                or self.floating_layout.focus_first(group=self)
+                or self.current_window
+                or self.layout.focus_first()
+            )
+
+            if win in self.tiled_windows:
+                self.tiled_windows.remove(win)
 
         # a notification may not have focus
         if hadfocus:
@@ -289,19 +315,27 @@ class _Group(CommandObject):
                 # already floating
                 pass
             else:
-                for i in self.layouts:
-                    i.remove(win)
+                # Remove from the tiled windows list if the window is not fullscreen
+                if not win.fullscreen:
+                    self.tiled_windows.remove(win)
+                    # Remove the window from the layout if it is not fullscreen
+                    for i in self.layouts:
+                        i.remove(win)
+                        if win is self.current_window:
+                            i.blur()
+                    self.floating_layout.add_client(win)
                     if win is self.current_window:
-                        i.blur()
-                self.floating_layout.add(win)
-                if win is self.current_window:
-                    self.floating_layout.focus(win)
+                        self.floating_layout.focus(win)
         else:
             self.floating_layout.remove(win)
             self.floating_layout.blur()
-            for i in self.layouts:
-                i.add(win)
-                if win is self.current_window:
+            # A window that was fullscreen should only be added if it was not a tiled window
+            if win not in self.tiled_windows:
+                for i in self.layouts:
+                    i.add_client(win)
+                self.tiled_windows.add(win)
+            if win is self.current_window:
+                for i in self.layouts:
                     i.focus(win)
         self.layout_all()
 
@@ -329,14 +363,12 @@ class _Group(CommandObject):
                     return i
         raise RuntimeError("Invalid selection: {}".format(name))
 
-    def cmd_setlayout(self, layout):
+    @expose_command()
+    def setlayout(self, layout):
         self.layout = layout
 
-    def cmd_info(self):
-        """Returns a dictionary of info for this group"""
-        return self.info()
-
-    def cmd_toscreen(self, screen=None, toggle=True):
+    @expose_command()
+    def toscreen(self, screen=None, toggle=False):
         """Pull a group to a specified screen.
 
         Parameters
@@ -402,13 +434,15 @@ class _Group(CommandObject):
     def get_next_group(self, skip_empty=False, skip_managed=False):
         return self._get_group(1, skip_empty, skip_managed)
 
-    def cmd_unminimize_all(self):
+    @expose_command()
+    def unminimize_all(self):
         """Unminimise all windows in this group"""
         for win in self.windows:
             win.minimized = False
         self.layout_all()
 
-    def cmd_next_window(self):
+    @expose_command()
+    def next_window(self):
         """
         Focus the next window in group.
 
@@ -419,16 +453,21 @@ class _Group(CommandObject):
         if not self.windows:
             return
         if self.current_window.floating:
-            nxt = self.floating_layout.focus_next(self.current_window) or \
-                self.layout.focus_first() or \
-                self.floating_layout.focus_first(group=self)
+            nxt = (
+                self.floating_layout.focus_next(self.current_window)
+                or self.layout.focus_first()
+                or self.floating_layout.focus_first(group=self)
+            )
         else:
-            nxt = self.layout.focus_next(self.current_window) or \
-                self.floating_layout.focus_first(group=self) or \
-                self.layout.focus_first()
+            nxt = (
+                self.layout.focus_next(self.current_window)
+                or self.floating_layout.focus_first(group=self)
+                or self.layout.focus_first()
+            )
         self.focus(nxt, True)
 
-    def cmd_prev_window(self):
+    @expose_command()
+    def prev_window(self):
         """
         Focus the previous window in group.
 
@@ -439,16 +478,21 @@ class _Group(CommandObject):
         if not self.windows:
             return
         if self.current_window.floating:
-            nxt = self.floating_layout.focus_previous(self.current_window) or \
-                self.layout.focus_last() or \
-                self.floating_layout.focus_last(group=self)
+            nxt = (
+                self.floating_layout.focus_previous(self.current_window)
+                or self.layout.focus_last()
+                or self.floating_layout.focus_last(group=self)
+            )
         else:
-            nxt = self.layout.focus_previous(self.current_window) or \
-                self.floating_layout.focus_last(group=self) or \
-                self.layout.focus_last()
+            nxt = (
+                self.layout.focus_previous(self.current_window)
+                or self.floating_layout.focus_last(group=self)
+                or self.layout.focus_last()
+            )
         self.focus(nxt, True)
 
-    def cmd_focus_back(self):
+    @expose_command()
+    def focus_back(self):
         """
         Focus the window that had focus before the current one got it.
 
@@ -463,7 +507,8 @@ class _Group(CommandObject):
         else:
             self.focus(win)
 
-    def cmd_focus_by_name(self, name):
+    @expose_command()
+    def focus_by_name(self, name):
         """
         Focus the first window with the given name. Do nothing if the name is
         not found.
@@ -473,7 +518,8 @@ class _Group(CommandObject):
                 self.focus(win)
                 break
 
-    def cmd_info_by_name(self, name):
+    @expose_command()
+    def info_by_name(self, name):
         """
         Get the info for the first window with the given name without giving it
         focus. Do nothing if the name is not found.
@@ -482,11 +528,40 @@ class _Group(CommandObject):
             if win.name == name:
                 return win.info()
 
-    def cmd_switch_groups(self, name):
-        """Switch position of current group with name"""
-        self.qtile.cmd_switch_groups(self.name, name)
+    @expose_command()
+    def focus_by_index(self, index: int) -> None:
+        """
+        Change to the window at the specified index in the current group.
+        """
+        windows = self.windows
+        if index < 0 or index > len(windows) - 1:
+            return
 
-    def cmd_set_label(self, label):
+        self.focus(windows[index])
+
+    @expose_command()
+    def swap_window_order(self, new_location: int) -> None:
+        """
+        Change the order of the current window within the current group.
+        """
+        if new_location < 0 or new_location > len(self.windows) - 1:
+            return
+
+        windows = self.windows
+        current_window_index = windows.index(self.current_window)
+
+        windows[current_window_index], windows[new_location] = (
+            windows[new_location],
+            windows[current_window_index],
+        )
+
+    @expose_command()
+    def switch_groups(self, name):
+        """Switch position of current group with name"""
+        self.qtile.switch_groups(self.name, name)
+
+    @expose_command()
+    def set_label(self, label):
         """
         Set the display name of current group to be used in GroupBox widget.
         If label is None, the name of the group is used as display name.
