@@ -28,7 +28,6 @@ widgets = {
     "Backlight": "backlight",
     "Battery": "battery",
     "BatteryIcon": "battery",
-    "BitcoinTicker": "bitcoin_ticker",
     "Bluetooth": "bluetooth",
     "CPU": "cpu",
     "CPUGraph": "graph",
@@ -40,12 +39,14 @@ widgets = {
     "Clock": "clock",
     "Cmus": "cmus",
     "Countdown": "countdown",
+    "CryptoTicker": "crypto_ticker",
     "CurrentLayout": "currentlayout",
     "CurrentLayoutIcon": "currentlayout",
     "CurrentScreen": "currentscreen",
     "DF": "df",
     "GenPollText": "generic_poll_text",
     "GenPollUrl": "generic_poll_text",
+    "GenPollCommand": "generic_poll_text",
     "GmailChecker": "gmail_checker",
     "GroupBox": "groupbox",
     "HDDBusyGraph": "graph",
@@ -57,6 +58,7 @@ widgets = {
     "KeyboardLayout": "keyboardlayout",
     "KhalCalendar": "khal_calendar",
     "LaunchBar": "launchbar",
+    "Load": "load",
     "Maildir": "maildir",
     "Memory": "memory",
     "MemoryGraph": "graph",
@@ -73,15 +75,18 @@ widgets = {
     "Prompt": "prompt",
     "PulseVolume": "pulse_volume",
     "QuickExit": "quick_exit",
+    "ScreenSplit": "screensplit",
     "Sep": "sep",
     "She": "she",
     "Spacer": "spacer",
+    "StatusNotifier": "statusnotifier",
     "StockTicker": "stock_ticker",
     "SwapGraph": "graph",
     "Systray": "systray",
     "TaskList": "tasklist",
     "TextBox": "textbox",
     "ThermalSensor": "sensors",
+    "ThermalZone": "thermal_zone",
     "Volume": "volume",
     "Wallpaper": "wallpaper",
     "WidgetBox": "widgetbox",
@@ -92,5 +97,4 @@ widgets = {
     "Wttr": "wttr",
 }
 
-__all__, __dir__, __getattr__ = lazify_imports(widgets, __package__,
-                                               fallback=make_error)
+__all__, __dir__, __getattr__ = lazify_imports(widgets, __package__, fallback=make_error)
